##############################################################################
# Copyright (c) 2004-2005 Black Rock City, LLC
# Copyright (c) 2004-2005 ObjectRealms, LLC
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
##############################################################################
"""
$Id: $
"""
from Testing import ZopeTestCase
from OFS.ObjectManager import BadRequestException
from Products.CMFPlone.tests import PloneTestCase

from AccessControl.SecurityManagement import newSecurityManager
from Products.Archetypes.Extensions.Install import install as install_archetypes

portal_name  = 'portal'
portal_owner = 'portal_owner'
default_user = 'unittest_admin'

ZopeTestCase.installProduct('ZCatalog')
ZopeTestCase.installProduct('MailHost', quiet=1)
ZopeTestCase.installProduct('CMFCore')
ZopeTestCase.installProduct('CMFDefault')
ZopeTestCase.installProduct('DCWorkflow')
ZopeTestCase.installProduct('CMFActionIcons')
ZopeTestCase.installProduct('CMFQuickInstallerTool')
ZopeTestCase.installProduct('CMFFormController')
ZopeTestCase.installProduct('ZCTextIndex')
ZopeTestCase.installProduct('CMFPlone')
ZopeTestCase.installProduct('Archetypes')
ZopeTestCase.installProduct('PortalTransforms')
ZopeTestCase.installProduct('MimetypesRegistry')
ZopeTestCase.installProduct('CMFMember')
ZopeTestCase.installProduct('TeamSpace')

# Make some members, roles
members = ['member_%s' %(x+1) for x in range(10)]

member_roles = ['Member']
manager_roles = ['Member', 'Manager']
reviewer_roles = ['Reviewer', 'Member']

makegroup = lambda members, status, roles: [((member, status), roles) for member in members]

member_map_1 =     makegroup(members[:1],  'activated', member_roles) # solo team
member_map_2 =     makegroup(members[:2],  'activated', member_roles) # team with inclusion
manager_map_1 =    makegroup(members[3:4], 'activated', manager_roles)
manager_map_2 =    makegroup(members[3:5], 'activated', manager_roles)
notmember = members[2]

##############################################################################
# Copyright (c) 2004-2005 Black Rock City, LLC
# Copyright (c) 2004-2005 ObjectRealms, LLC
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
##############################################################################
"""

MEMBER KEY:
M: manager
m: member
R: reviewer
N: nonmember

[teams>]    TEAM_1  TEAM_2  TEAM_3
member_1    m       m
member_2            m
member_3                            < member_3 is the token nonmember
member_4    m Ma    m Ma
member_5            m Ma
"""

from TeamDataBag import TeamDataBag

# create a couple of teams with managers

t1_map = member_map_1 + manager_map_1
t2_map = member_map_2 + manager_map_2

TEAM_1 = TeamDataBag('team_1', t1_map)
TEAM_2 = TeamDataBag('team_2', t2_map)

class TeamTestCase(PloneTestCase.PloneTestCase):

    def installProducts(self, products=[]):
        self.portal.portal_quickinstaller.installProducts(products=products, stoponerror=1)

    def afterSetUp( self ):
        self.loginPortalOwner()
        install_archetypes(self.portal)
        self.installProducts(products=['CMFMember'])
        self.portal.cmfmember_control.upgrade(swallow_errors=0)
        self.installProducts(products=['TeamSpace'])
        self._refreshSkinData()
        self.tool = self.portal.portal_teams
        self.uf = self.portal.acl_users

    def addAndPopulateTeam(self, teamData, add_roles=0):
        pm_tool = self.portal.portal_membership
        wf_tool = self.portal.portal_workflow

        self.loginPortalOwner()

        for m in teamData.getMemberIds():
            if not hasattr(self.portal.portal_memberdata, m):
              try:
                pm_tool.addMember( m,
                                   m,
                                   ('Member',),
                                   tuple(),
                                   properties={'email':'%s@example.com'%m} )
              except:
                import pdb, sys
                e, m, tb = sys.exc_info()
                pdb.post_mortem(tb)

            my_member = pm_tool.getMemberById(m)
            if wf_tool.getInfoFor(my_member, 'review_state') == 'new':
                wf_tool.doActionFor(my_member, 'migrate')

        tmtool = self.tool
        tmtool.invokeFactory("Team", teamData.id)
        team = tmtool.getTeamById(teamData.id)
        [team.addMember(member_id) for member_id in teamData.getMemberIds()]

        # add roles to memberships
        memberships = [(x.id, x) for x in team.getMemberships()]
        roles = teamData.rolesDict()
        for m_id, m_ship in memberships:
            m_ship.editTeamRoles(roles[m_id])

    def setStatusForMembership(self, teamData):
        wf_tool = self.portal.portal_workflow
        team = self.tool.getTeamById(teamData.id)
        m_ships  =  dict([ ( x.id, x ) for x  in team.getMemberships()])
        m_stati = teamData.statusDict()

        setStatus = wf_tool.setStatusOf
        getStatus = wf_tool.getStatusOf

        ### is there a multiple workflow case here?
        ### kt - potentially yes, we try to defer to a set of states applicable
        ### to any workflow and an explicit workflow id attribute on the membership
        ### to identify a workflow
        #workflowId = lambda x: wf_tool.getWorkflowsFor(x)[0]

        for m_id in m_stati.keys():
            m_ship = m_ships[m_id]
            wf_tool.doActionFor(m_ship, 'activate')


